@@ -56,11 +56,14 @@ import subprocess
 import random
 import traceback
 import string
+import sys
 import os
 import sqlite3
 import time
 import shutil
 import threading
+import traceback
+import re
 from contextlib import contextmanager
 
 
@@ -202,6 +205,9 @@ class program(object):
         """
         if not(isinstance(ex,Execution)):
             raise ValueError("First argument to program " + self.gen_args.__name__ + " must be an Execution.")
+        elif ex.id != None:
+            raise SyntaxError("Program being called on an execution that has already terminated.")
+
         if kwargs.has_key('stdout'):
             stdout = open(kwargs['stdout'],'w')
             kwargs.pop('stdout')
@@ -282,6 +288,8 @@ class program(object):
         """
         if not(isinstance(ex,Execution)):
             raise ValueError("First argument to a program must be an Execution.")
+        elif ex.id != None:
+            raise SyntaxError("Program being called on an execution that has already terminated.")
 
         if kwargs.has_key('via'):
             via = kwargs['via']
@@ -375,10 +383,9 @@ class program(object):
         """Method called by ``nonblocking`` to run via LSF."""
         if not(isinstance(ex,Execution)):
             raise ValueError("First argument to a program must be an Execution.")
-        d = self.gen_args(*args, **kwargs)
 
         if kwargs.has_key('stdout'):
-            stdout = open(kwargs['stdout'],'w')
+            stdout = kwargs['stdout']
             kwargs.pop('stdout')
             load_stdout = False
         else:
@@ -386,15 +393,22 @@ class program(object):
             load_stdout = True
 
         if kwargs.has_key('stderr'):
-            stderr = open(kwargs['stderr'],'w')
+            stderr = kwargs['stderr']
             kwargs.pop('stderr')
             load_stderr = False
         else:
             stderr = unique_filename_in(ex.working_directory)
             load_stderr = True
 
-        cmds = ["bsub","-cwd",ex.remote_working_directory,"-o",stdout,"-e",stderr,
-                "-K","-r"] + d["arguments"]
+        d = self.gen_args(*args, **kwargs)
+
+        # Jacques Rougemont figured out the following syntax that works in
+        # both bash and tcsh.
+        remote_cmd = " ".join(d["arguments"])
+        remote_cmd += " > "+stdout
+        remote_cmd = " ( "+remote_cmd+" ) >& "+stderr
+        cmds = ["bsub","-cwd",ex.remote_working_directory,"-o","/dev/null",
+                "-e","/dev/null","-K","-r",remote_cmd]
         class Future(object):
             def __init__(self):
                 self.program_output = None
@@ -406,34 +420,13 @@ class program(object):
         f = Future()
         v = threading.Event()
         def g():
-<<<<<<< HEAD
-            nullout = open(os.path.devnull, 'w')
-            sp = subprocess.Popen(cmds, bufsize=-1, stdout=nullout, stderr=nullout)
-            return_code = sp.wait()
-            stdout = None
-            stderr = None
-            while not(os.path.exists(os.path.join(ex.working_directory, stdout_filename)) and
-                      os.path.exists(os.path.join(ex.working_directory, stderr_filename))):
-                pass # We need to wait until the files actually show up
-            with open(os.path.join(ex.working_directory,stdout_filename), 'r') as fo:
-                stdout = fo.readlines()
-            with open(os.path.join(ex.working_directory,stderr_filename), 'r') as fe:
-                stderr = fe.readlines()
-            f.program_output = ProgramOutput(return_code, sp.pid,
-                                             cmds, stdout, stderr)
-            if return_code == 0:
-                z = d["return_value"]
-                if callable(z):
-                    f.return_value = z(f.program_output)
-=======
             try:
                 nullout = open(os.path.devnull, 'w')
-                sp = subprocess.Popen(cmds, bufsize=-1, stdout=nullout, stderr=nullout)
+                sp = subprocess.Popen(cmds, bufsize=-1, stdout=nullout, 
+                                      stderr=nullout)
                 return_code = sp.wait()
                 while not(os.path.exists(os.path.join(ex.working_directory,
-                                                      stdout)) and
-                          os.path.exists(os.path.join(ex.working_directory,
-                                                      stderr))):
+                                                      stdout))):
                     pass # We need to wait until the files actually show up
                 if load_stdout:
                     with open(os.path.join(ex.working_directory,stdout), 'r') as fo:
@@ -441,10 +434,11 @@ class program(object):
                 else:
                     stdout_value = None
 
+                while not(os.path.exists(os.path.join(ex.working_directory,stderr))):
+                    pass # We need to wait until the files actually show up
                 if load_stderr:
                     with open(os.path.join(ex.working_directory,stderr), 'r') as fe:
-                        stderr = fe.readlines()
->>>>>>> 453ace62cb6bc6fc25447d3e2566b82cc05c2f8f
+                        stderr_value = fe.readlines()
                 else:
                     stderr_value = None
 
@@ -491,6 +485,13 @@ class Execution(object):
         self.started_at = int(time.time())
         self.finished_at = None
         self.id = None
+    def path_to_file(self, id_or_alias):
+        """Fetch the path to *id_or_alias* in the attached LIMS."""
+        if self.lims == None:
+            raise ValueError("Cannot use path_to_file; no attached LIMS.")
+        else:
+            return self.lims.path_to_file(id_or_alias)
+
     def report(self, program):
         """Add a ProgramOutput object to the execution.
 
@@ -603,9 +604,11 @@ def execution(lims = None, description="", remote_working_directory=None):
     exception_string = None
     try:
         yield ex
-    except Exception, e:
-        exception_string = ''.join(traceback.format_exception_only(e.__class__, str(e)))
-        raise e
+    except:
+        (exc_type, exc_value, exc_traceback) = sys.exc_info()
+        exception_string = ''.join(traceback.format_exception(exc_type, exc_value,
+                                                            exc_traceback))
+        raise
     finally:
         ex.finish()
         try:
@@ -614,6 +617,8 @@ def execution(lims = None, description="", remote_working_directory=None):
         finally:
             os.chdir("..")
             shutil.rmtree(ex.working_directory, ignore_errors=True)
+            cleaned_up = True
+        assert(cleaned_up)
 
 
 class MiniLIMS(object):
@@ -948,7 +953,7 @@ class MiniLIMS(object):
                 fileids[filename] = self._insert_file(ex, exid, filename, description)
 
                 if alias != None:
-                    self.add_alias(thisid, alias)
+                    self.add_alias(fileids[filename], alias)
 
                 if associate_to_id != None or associate_to_filename != None:
                     if template == None:
